// SBPropertyChangerV1, 17 Feb 2020, SeriousBarbie AT googlemail.com
// Changes the given properties of the *WantedActor*.
class SBPropertyChanger expands Triggers;


var(PropertyChanger) name WantedActor;
var(PropertyChanger) String Properties[8]; // Properties to change
var(PropertyChanger) String PropertyValues[8]; // their values
var(PropertyChanger) bool bTriggerOnceOnly;
var(PropertyChanger) const enum EWantedActorSearchBy // Should the *WantedActor* searched by Name or by Tag?
{
	WASB_Name,
	WASB_Tag
} WantedActorSearchBy;


function PostBeginPlay() {
	Super.PostBeginPlay();
	if (WantedActor == '')
	{
		warn("'WantedActor' must not be NONE!" @ self @ "is terminating now.");
		Destroy();
	}
}



function Trigger(actor Other, pawn EventInstigator) {
	Instigator = EventInstigator;
	gotostate('ChangeProperties');
}



function bool FindActorByName(name ActorName, out Actor FoundActor, optional name ActorTag) {
local Actor A;

	foreach AllActors(class'Actor', A, ActorTag)
		if (A.Name == ActorName)
		{
			FoundActor = A;
			return true;
		}
	return false;
}


function bool SpecialPropertyHandling(Actor A, string PropName, string PropValue, out int bError) {
	if (PropName ~= "CollisionRadius")
	{
		A.SetCollisionSize(float(PropValue), A.CollisionHeight);
		bError = int(A.CollisionRadius != float(PropValue));
		return true;
	}
	if (PropName ~= "CollisionHeight")
	{
		A.SetCollisionSize(A.CollisionRadius, float(PropValue));
		bError = int(A.CollisionHeight != float(PropValue));
		return true;
	}
	if (PropName ~= "bCollideActors")
	{
		A.SetCollision(bool(PropValue));
		bError = int(A.bCollideActors != bool(PropValue));
		return true;
	}
	if (PropName ~= "bBlockActors")
	{
		A.SetCollision(, bool(PropValue));
		bError = int(A.bBlockActors != bool(PropValue));
		return true;
	}
	if (PropName ~= "bBlockPlayers")
	{
		A.SetCollision(, , bool(PropValue));
		bError = int(A.bBlockPlayers != bool(PropValue));
		return true;
	}
	return false;
}


state ChangeProperties {


	function bool TreatActor(Actor A) {
	// returns TRUE if change was successful
	local int i;
	local int bError;

	for (i=0; i < ArrayCount(Properties); i++)
		if (Properties[i] != "")
		{
			if ( ! SpecialPropertyHandling(A, Properties[i], PropertyValues[i], bError))
			{
				A.SetPropertyText(Properties[i], PropertyValues[i]);
				if (A.GetPropertyText(Properties[i]) != PropertyValues[i])
					bError = int( ! (A.GetPropertyText(Properties[i]) == "None" && PropertyValues[i] == "")); // special equal case: "None == ''"
				else
					bError = int(false);
			}
			if (bool(bError))
				warn("Setting property '" $ Properties[i] $ "' with value '" $ PropertyValues[i] $ "' failed, current value=" $ A.GetPropertyText(Properties[i]));
		}
	}



	function TreatActors() {
	local Actor A;
	local bool bActorFound;

		if (WantedActorSearchBy == WASB_Name)
		{
			if (FindActorByName(WantedActor, A))
				TreatActor(A);
			else
				warn("Actor with Name='" $ WantedActor $ "' not found");
		}
		else
		{
			bActorFound = false;
			ForEach AllActors(Class'Actor', A, WantedActor)
			{
				TreatActor(A);
				bActorFound = true;
			}
			if ( ! bActorFound)
				warn("No Actor with tag '" $ WantedActor $ "' found");
		}
		if ( ! bTriggerOnceOnly)
			enable('Trigger');
	}
begin:
	disable('Trigger');
//	sleep(Delay);
	TreatActors();
}


defaultproperties {
	bCollideActors=false
	CollisionHeight=0
	CollisionRadius=0

}


